require_relative 'collection_probe'
require_relative 'etcd_probe'

module Philae
  class EtcdClusterProbe < CollectionProbe
    attr_reader :name, :etcd_probes

    # @param [Integer] read_timeout Timeout in second for the HTTP request
    def initialize(name, endpoints, read_timeout: 1, cacert: nil, ssl_cert: nil, ssl_key: nil)
      raise ArgumentError, 'empty endpoints' if endpoints.nil? || endpoints.empty?

      @etcd_probes = endpoints.map do |endpoint|
        endpoint_uri = URI(endpoint)
        next EtcdProbe.new(
          "#{name}-#{endpoint_uri.host}-#{endpoint_uri.port}",
          endpoint_uri.host,
          endpoint_uri.port,
          read_timeout: read_timeout, cacert: cacert,
          ssl_cert: ssl_cert, ssl_key: ssl_key,
        )
      end

      @name = name
    end

    def probes
      return @etcd_probes
    end
  end
end
