require_relative 'collection_probe.rb'
require_relative 'nsq_probe.rb'

module Philae
  class NSQClusterProbe < CollectionProbe
    DEFAULT_HTTP_PORT = 4152

    attr_reader :name, :nsq_probes

    def self.new_from_env(name, opts = {})
      if ENV['NSQD_HOSTS'].nil? || ENV['NSQD_HTTP_PORT'].nil?
        raise ArgumentError, 'no NSQD_HOSTS and NSQD_HTTP_PORT defined'
      end

      tls_context = nil
      if ENV['NSQD_TLS'] == 'true'
        tls_context = {
          cert: ENV['NSQD_TLS_CERT'],
          key: ENV['NSQD_TLS_KEY'],
          ca: ENV['NSQD_TLS_CACERT'],
        }
      end

      http_hosts = ENV['NSQD_HOSTS'].split(',').map do |host|
        host.split(':')[0] + ":#{ENV['NSQD_HTTP_PORT']}"
      end

      new(name, http_hosts, tls_context, opts)
    end

    def initialize(name, hosts, tls_context = nil, opts = {})
      raise ArgumentError, 'should have at least one host' if hosts.nil? || hosts.empty?

      @name = name
      @nsq_probes = hosts.map do |hostport|
        host, port = hostport.split(':')
        port = DEFAULT_HTTP_PORT if port.nil?
        next Philae::NSQProbe.new("#{name}-#{host}-#{port}", host, port, tls_context, opts)
      end
    end

    def probes
      @nsq_probes
    end
  end
end
