require 'pg'

# Philae::PostgreSQLProbe tries to connect to the given connection string and
# ping the database. The connection string must be in the form
# `postgres://[username:password@]host1[:port1][/[database][?options]]`
module Philae
  class PostgreSQLProbe
    attr_reader :name

    def initialize(name, connection_string)
      @name = name
      @connection_string = connection_string
    end

    def check
      begin
        res = PG::Connection.ping(@connection_string)
      rescue
        return { healthy: false, comment: 'Unable to contact PostgreSQL' }
      end
      return { healthy: true, comment: '' } if res == PG::PQPING_OK

      # https://deveiate.org/code/pg/PG/Connection.html#method-c-ping
      reasons = {
        PG::PQPING_REJECT => 'server is alive but rejecting connections',
        PG::PQPING_NO_RESPONSE => 'could not establish connection',
        PG::PQPING_NO_ATTEMPT => 'connection not attempted (bad params)',
      }
      return { healthy: false, comment: "Ping to PostgreSQL failed with the following reason: #{reasons[res]}" }
    end
  end
end
